// Copyright (c) 2001  Per M.A. Bothner and Brainfood Inc.
// This is free software;  for terms and warranty disclaimer see ./COPYING.

package gnu.kawa.servlet;
import java.net.MalformedURLException;
import java.net.URL;
import javax.servlet.*;
import javax.servlet.http.*;
import gnu.mapping.*;

public class ServletCallContext extends CallContext
{
  public HttpServletRequest request;
  public HttpServletResponse response;
  public HttpServlet servlet;

  static ServletCallContext getServletCallContext ()
  {
    return (ServletCallContext) CallContext.getOnlyInstance();
  }

  public static HttpServletRequest getRequest ()
  {
    return getServletCallContext().request;
  }

  public static HttpServletResponse getResponse ()
  {
    return getServletCallContext().response;
  }

  public static HttpServlet getServlet ()
  {
    return getServletCallContext().servlet;
  }

  public static ServletConfig getServletConfig ()
  {
    return getServletCallContext().servlet.getServletConfig();
  }

  public static ServletContext getServletContext ()
  {
    return getServletCallContext().servlet.getServletConfig().getServletContext();
  }
  

  static final String DIR_SEPARATOR = "/";


  /**
   * Returns an absolute URL given path relative to the servlet context
   * 
   */
  public URL getResource( String name ) 
    throws MalformedURLException
  {
    URL url = null;

    String spath = request.getServletPath();
    StringBuffer fullUrl = request.getRequestURL();

    // path relative to current file
    if (! name.startsWith( DIR_SEPARATOR ))
    {
      int lastDirIdx = spath.lastIndexOf( DIR_SEPARATOR ); 
      if (lastDirIdx < 0) lastDirIdx = 0;
      int baseUrlEnd = fullUrl.indexOf(spath) + lastDirIdx + 1;
      fullUrl.delete( baseUrlEnd, fullUrl.length() );
      fullUrl.append( name );
    }
    // path relative to servlet context 
    else
    {
      fullUrl.delete( fullUrl.indexOf(spath), fullUrl.length() );
      fullUrl.append( name ).toString();
    }
    url = new URL( fullUrl.toString() );
    return url;
  }
  
}
